This is the mail archive of the gdb-patches@sourceware.org mailing list for the GDB project.


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]
Other format: [Raw text]

Frame unwindig updates for m68k port


This updates the frame unwinding for the m68k port.  Tested on
m68k-linux and checked in.

Andreas.

2008-05-05  Andreas Schwab  <schwab@suse.de>

	Update m68k port for unwinder changes.
	* m68k-tdep.c (m68k_frame_cache): Expect this_frame.
	(m68k_frame_this_id, m68k_frame_prev_register): Update signature.
	(m68k_frame_unwind): Use default_frame_sniffer.
	(m68k_frame_sniffer): Remove.
	(m68k_frame_base_address): Expect this_frame.
	(m68k_dummy_id): Renamed from m68k_unwind_dummy_id.  Expect
	this_frame.
	(m68k_gdbarch_init): Use set_gdbarch_dummy_id,
	dwarf2_append_unwinders, and frame_unwind_append_unwinder.
	* m68klinux-tdep.c (m68k_linux_pc_in_sigtramp): Expect frame_info
	parameter instead of pc value.
	(m68k_linux_get_sigtramp_info, m68k_linux_sigtramp_frame_cache):
	Expect this_frame.
	(m68k_linux_sigtramp_frame_this_id)
	(m68k_linux_sigtramp_frame_prev_register)
	(m68k_linux_sigtramp_frame_sniffer): Update signature.
	(m68k_linux_sigtramp_frame_unwind): Use
	m68k_linux_sigtramp_frame_sniffer.
	(m68k_linux_init_abi): Use frame_unwind_append_unwinder.

	* m68klinux-nat.c (store_register): Fix typo.

Index: m68k-tdep.c
===================================================================
RCS file: /cvs/src/src/gdb/m68k-tdep.c,v
retrieving revision 1.134
diff -u -a -p -r1.134 m68k-tdep.c
--- m68k-tdep.c	22 Apr 2008 11:03:42 -0000	1.134
+++ m68k-tdep.c	5 May 2008 20:43:20 -0000
@@ -838,7 +838,7 @@ m68k_unwind_pc (struct gdbarch *gdbarch,
 /* Normal frames.  */
 
 static struct m68k_frame_cache *
-m68k_frame_cache (struct frame_info *next_frame, void **this_cache)
+m68k_frame_cache (struct frame_info *this_frame, void **this_cache)
 {
   struct m68k_frame_cache *cache;
   gdb_byte buf[4];
@@ -859,7 +859,7 @@ m68k_frame_cache (struct frame_info *nex
      They (usually) share their frame pointer with the frame that was
      in progress when the signal occurred.  */
 
-  frame_unwind_register (next_frame, M68K_FP_REGNUM, buf);
+  get_frame_register (this_frame, M68K_FP_REGNUM, buf);
   cache->base = extract_unsigned_integer (buf, 4);
   if (cache->base == 0)
     return cache;
@@ -867,10 +867,10 @@ m68k_frame_cache (struct frame_info *nex
   /* For normal frames, %pc is stored at 4(%fp).  */
   cache->saved_regs[M68K_PC_REGNUM] = 4;
 
-  cache->pc = frame_func_unwind (next_frame, NORMAL_FRAME);
+  cache->pc = get_frame_func (this_frame);
   if (cache->pc != 0)
-    m68k_analyze_prologue (get_frame_arch (next_frame), cache->pc,
-			   frame_pc_unwind (next_frame), cache);
+    m68k_analyze_prologue (get_frame_arch (this_frame), cache->pc,
+			   get_frame_pc (this_frame), cache);
 
   if (cache->locals < 0)
     {
@@ -882,7 +882,7 @@ m68k_frame_cache (struct frame_info *nex
 	 frame by looking at the stack pointer.  For truly "frameless"
 	 functions this might work too.  */
 
-      frame_unwind_register (next_frame, M68K_SP_REGNUM, buf);
+      get_frame_register (this_frame, M68K_SP_REGNUM, buf);
       cache->base = extract_unsigned_integer (buf, 4) + cache->sp_offset;
     }
 
@@ -900,10 +900,10 @@ m68k_frame_cache (struct frame_info *nex
 }
 
 static void
-m68k_frame_this_id (struct frame_info *next_frame, void **this_cache,
+m68k_frame_this_id (struct frame_info *this_frame, void **this_cache,
 		    struct frame_id *this_id)
 {
-  struct m68k_frame_cache *cache = m68k_frame_cache (next_frame, this_cache);
+  struct m68k_frame_cache *cache = m68k_frame_cache (this_frame, this_cache);
 
   /* This marks the outermost frame.  */
   if (cache->base == 0)
@@ -913,70 +913,37 @@ m68k_frame_this_id (struct frame_info *n
   *this_id = frame_id_build (cache->base + 8, cache->pc);
 }
 
-static void
-m68k_frame_prev_register (struct frame_info *next_frame, void **this_cache,
-			  int regnum, int *optimizedp,
-			  enum lval_type *lvalp, CORE_ADDR *addrp,
-			  int *realnump, gdb_byte *valuep)
+static struct value *
+m68k_frame_prev_register (struct frame_info *this_frame, void **this_cache,
+			  int regnum)
 {
-  struct m68k_frame_cache *cache = m68k_frame_cache (next_frame, this_cache);
+  struct m68k_frame_cache *cache = m68k_frame_cache (this_frame, this_cache);
 
   gdb_assert (regnum >= 0);
 
   if (regnum == M68K_SP_REGNUM && cache->saved_sp)
-    {
-      *optimizedp = 0;
-      *lvalp = not_lval;
-      *addrp = 0;
-      *realnump = -1;
-      if (valuep)
-	{
-	  /* Store the value.  */
-	  store_unsigned_integer (valuep, 4, cache->saved_sp);
-	}
-      return;
-    }
+    return frame_unwind_got_constant (this_frame, regnum, cache->saved_sp);
 
   if (regnum < M68K_NUM_REGS && cache->saved_regs[regnum] != -1)
-    {
-      *optimizedp = 0;
-      *lvalp = lval_memory;
-      *addrp = cache->saved_regs[regnum];
-      *realnump = -1;
-      if (valuep)
-	{
-	  /* Read the value in from memory.  */
-	  read_memory (*addrp, valuep,
-		       register_size (get_frame_arch (next_frame), regnum));
-	}
-      return;
-    }
+    return frame_unwind_got_memory (this_frame, regnum,
+				    cache->saved_regs[regnum]);
 
-  *optimizedp = 0;
-  *lvalp = lval_register;
-  *addrp = 0;
-  *realnump = regnum;
-  if (valuep)
-    frame_unwind_register (next_frame, (*realnump), valuep);
+  return frame_unwind_got_register (this_frame, regnum, regnum);
 }
 
 static const struct frame_unwind m68k_frame_unwind =
 {
   NORMAL_FRAME,
   m68k_frame_this_id,
-  m68k_frame_prev_register
+  m68k_frame_prev_register,
+  NULL,
+  default_frame_sniffer
 };
-
-static const struct frame_unwind *
-m68k_frame_sniffer (struct frame_info *next_frame)
-{
-  return &m68k_frame_unwind;
-}
 
 static CORE_ADDR
-m68k_frame_base_address (struct frame_info *next_frame, void **this_cache)
+m68k_frame_base_address (struct frame_info *this_frame, void **this_cache)
 {
-  struct m68k_frame_cache *cache = m68k_frame_cache (next_frame, this_cache);
+  struct m68k_frame_cache *cache = m68k_frame_cache (this_frame, this_cache);
 
   return cache->base;
 }
@@ -990,16 +957,14 @@ static const struct frame_base m68k_fram
 };
 
 static struct frame_id
-m68k_unwind_dummy_id (struct gdbarch *gdbarch, struct frame_info *next_frame)
+m68k_dummy_id (struct gdbarch *gdbarch, struct frame_info *this_frame)
 {
-  gdb_byte buf[4];
   CORE_ADDR fp;
 
-  frame_unwind_register (next_frame, M68K_FP_REGNUM, buf);
-  fp = extract_unsigned_integer (buf, 4);
+  fp = get_frame_register_unsigned (this_frame, M68K_FP_REGNUM);
 
   /* See the end of m68k_push_dummy_call.  */
-  return frame_id_build (fp + 8, frame_pc_unwind (next_frame));
+  return frame_id_build (fp + 8, get_frame_pc (this_frame));
 }
 
 
@@ -1238,11 +1203,11 @@ m68k_gdbarch_init (struct gdbarch_info i
   tdep->struct_return = reg_struct_return;
 
   /* Frame unwinder.  */
-  set_gdbarch_unwind_dummy_id (gdbarch, m68k_unwind_dummy_id);
+  set_gdbarch_dummy_id (gdbarch, m68k_dummy_id);
   set_gdbarch_unwind_pc (gdbarch, m68k_unwind_pc);
 
   /* Hook in the DWARF CFI frame unwinder.  */
-  frame_unwind_append_sniffer (gdbarch, dwarf2_frame_sniffer);
+  dwarf2_append_unwinders (gdbarch);
 
   frame_base_set_default (gdbarch, &m68k_frame_base);
 
@@ -1255,7 +1220,7 @@ m68k_gdbarch_init (struct gdbarch_info i
   if (tdep->jb_pc >= 0)
     set_gdbarch_get_longjmp_target (gdbarch, m68k_get_longjmp_target);
 
-  frame_unwind_append_sniffer (gdbarch, m68k_frame_sniffer);
+  frame_unwind_append_unwinder (gdbarch, &m68k_frame_unwind);
 
   if (tdesc_data)
     tdesc_use_registers (gdbarch, info.target_desc, tdesc_data);
Index: m68klinux-nat.c
===================================================================
RCS file: /cvs/src/src/gdb/m68klinux-nat.c,v
retrieving revision 1.46
diff -u -a -p -r1.46 m68klinux-nat.c
--- m68klinux-nat.c	18 Feb 2008 19:58:27 -0000	1.46
+++ m68klinux-nat.c	5 May 2008 20:43:20 -0000
@@ -179,7 +179,7 @@ old_fetch_inferior_registers (struct reg
 static void
 store_register (const struct regcache *regcache, int regno)
 {
-  struct gdbarch *gdbarch = reg_regcache_arch (regcache);
+  struct gdbarch *gdbarch = get_regcache_arch (regcache);
   /* This isn't really an address.  But ptrace thinks of it as one.  */
   CORE_ADDR regaddr;
   char mess[128];		/* For messages */
Index: m68klinux-tdep.c
===================================================================
RCS file: /cvs/src/src/gdb/m68klinux-tdep.c,v
retrieving revision 1.28
diff -u -a -p -r1.28 m68klinux-tdep.c
--- m68klinux-tdep.c	13 Mar 2008 12:22:13 -0000	1.28
+++ m68klinux-tdep.c	5 May 2008 20:43:20 -0000
@@ -58,18 +58,19 @@
    /* moveq #82,d0; notb d0; trap #0 */					\
    || (insn1 == 0x70524600 && (insn2 >> 16) == 0x4e40))
 
-/* Return non-zero if PC points into the signal trampoline.  For the
-   sake of m68k_linux_get_sigtramp_info we also distinguish between
+/* Return non-zero if THIS_FRAME corresponds to a signal trampoline.  For
+   the sake of m68k_linux_get_sigtramp_info we also distinguish between
    non-RT and RT signal trampolines.  */
 
 static int
-m68k_linux_pc_in_sigtramp (CORE_ADDR pc, char *name)
+m68k_linux_pc_in_sigtramp (struct frame_info *this_frame)
 {
   CORE_ADDR sp;
-  char buf[12];
+  gdb_byte buf[12];
   unsigned long insn0, insn1, insn2;
+  CORE_ADDR pc = get_frame_pc (this_frame);
 
-  if (target_read_memory (pc - 4, buf, sizeof (buf)))
+  if (safe_frame_unwind_memory (this_frame, pc - 4, buf, sizeof (buf)))
     return 0;
   insn1 = extract_unsigned_integer (buf + 4, 4);
   insn2 = extract_unsigned_integer (buf + 8, 4);
@@ -219,68 +220,65 @@ m68k_linux_inferior_created (struct targ
 }
 
 static struct m68k_linux_sigtramp_info
-m68k_linux_get_sigtramp_info (struct frame_info *next_frame)
+m68k_linux_get_sigtramp_info (struct frame_info *this_frame)
 {
   CORE_ADDR sp;
-  char buf[4];
   struct m68k_linux_sigtramp_info info;
 
   if (target_is_uclinux == -1)
     {
       /* Determine whether we are running on a uClinux or normal GNU/Linux
          target so we can use the correct sigcontext layouts.  */
-    
       CORE_ADDR dummy;
-    
+
       target_is_uclinux
-        = target_auxv_search (&current_target, AT_NULL, &dummy) > 0
-          && target_auxv_search (&current_target, AT_PAGESZ, &dummy) == 0;
+        = (target_auxv_search (&current_target, AT_NULL, &dummy) > 0
+	   && target_auxv_search (&current_target, AT_PAGESZ, &dummy) == 0);
     }
 
-  frame_unwind_register (next_frame, M68K_SP_REGNUM, buf);
-  sp = extract_unsigned_integer (buf, 4);
+  sp = get_frame_register_unsigned (this_frame, M68K_SP_REGNUM);
 
   /* Get sigcontext address, it is the third parameter on the stack.  */
   info.sigcontext_addr = read_memory_unsigned_integer (sp + 8, 4);
 
-  if (m68k_linux_pc_in_sigtramp (frame_pc_unwind (next_frame), 0) == 2)
+  if (m68k_linux_pc_in_sigtramp (this_frame) == 2)
     info.sc_reg_offset = m68k_linux_ucontext_reg_offset;
   else
-    info.sc_reg_offset
-      = target_is_uclinux ? m68k_uclinux_sigcontext_reg_offset
-			  : m68k_linux_sigcontext_reg_offset;
+    info.sc_reg_offset = (target_is_uclinux
+			  ? m68k_uclinux_sigcontext_reg_offset
+			  : m68k_linux_sigcontext_reg_offset);
   return info;
 }
 
 /* Signal trampolines.  */
 
 static struct trad_frame_cache *
-m68k_linux_sigtramp_frame_cache (struct frame_info *next_frame,
+m68k_linux_sigtramp_frame_cache (struct frame_info *this_frame,
 				 void **this_cache)
 {
   struct frame_id this_id;
   struct trad_frame_cache *cache;
-  struct gdbarch_tdep *tdep = gdbarch_tdep (get_frame_arch (next_frame));
+  struct gdbarch_tdep *tdep = gdbarch_tdep (get_frame_arch (this_frame));
   struct m68k_linux_sigtramp_info info;
-  char buf[4];
+  gdb_byte buf[4];
   int i;
 
   if (*this_cache)
     return *this_cache;
 
-  cache = trad_frame_cache_zalloc (next_frame);
+  cache = trad_frame_cache_zalloc (this_frame);
 
   /* FIXME: cagney/2004-05-01: This is is long standing broken code.
      The frame ID's code address should be the start-address of the
      signal trampoline and not the current PC within that
      trampoline.  */
-  frame_unwind_register (next_frame, M68K_SP_REGNUM, buf);
+  get_frame_register (this_frame, M68K_SP_REGNUM, buf);
   /* See the end of m68k_push_dummy_call.  */
   this_id = frame_id_build (extract_unsigned_integer (buf, 4) - 4 + 8,
-			    frame_pc_unwind (next_frame));
+			    get_frame_pc (this_frame));
   trad_frame_set_id (cache, this_id);
 
-  info = m68k_linux_get_sigtramp_info (next_frame);
+  info = m68k_linux_get_sigtramp_info (this_frame);
 
   for (i = 0; i < M68K_NUM_REGS; i++)
     if (info.sc_reg_offset[i] != -1)
@@ -292,50 +290,43 @@ m68k_linux_sigtramp_frame_cache (struct 
 }
 
 static void
-m68k_linux_sigtramp_frame_this_id (struct frame_info *next_frame,
+m68k_linux_sigtramp_frame_this_id (struct frame_info *this_frame,
 				   void **this_cache,
 				   struct frame_id *this_id)
 {
   struct trad_frame_cache *cache =
-    m68k_linux_sigtramp_frame_cache (next_frame, this_cache);
+    m68k_linux_sigtramp_frame_cache (this_frame, this_cache);
   trad_frame_get_id (cache, this_id);
 }
 
-static void
-m68k_linux_sigtramp_frame_prev_register (struct frame_info *next_frame,
+static struct value *
+m68k_linux_sigtramp_frame_prev_register (struct frame_info *this_frame,
 					 void **this_cache,
-					 int regnum, int *optimizedp,
-					 enum lval_type *lvalp,
-					 CORE_ADDR *addrp,
-					 int *realnump, gdb_byte *valuep)
+					 int regnum)
 {
   /* Make sure we've initialized the cache.  */
   struct trad_frame_cache *cache =
-    m68k_linux_sigtramp_frame_cache (next_frame, this_cache);
-  trad_frame_get_register (cache, next_frame, regnum, optimizedp, lvalp,
-			   addrp, realnump, valuep);
+    m68k_linux_sigtramp_frame_cache (this_frame, this_cache);
+  return trad_frame_get_register (cache, this_frame, regnum);
+}
+
+static int
+m68k_linux_sigtramp_frame_sniffer (const struct frame_unwind *self,
+				   struct frame_info *this_frame,
+				   void **this_prologue_cache)
+{
+  return m68k_linux_pc_in_sigtramp (this_frame);
 }
 
 static const struct frame_unwind m68k_linux_sigtramp_frame_unwind =
 {
   SIGTRAMP_FRAME,
   m68k_linux_sigtramp_frame_this_id,
-  m68k_linux_sigtramp_frame_prev_register
+  m68k_linux_sigtramp_frame_prev_register,
+  NULL,
+  m68k_linux_sigtramp_frame_sniffer
 };
 
-static const struct frame_unwind *
-m68k_linux_sigtramp_frame_sniffer (struct frame_info *next_frame)
-{
-  CORE_ADDR pc = frame_pc_unwind (next_frame);
-  char *name;
-
-  find_pc_partial_function (pc, &name, NULL, NULL);
-  if (m68k_linux_pc_in_sigtramp (pc, name))
-    return &m68k_linux_sigtramp_frame_unwind;
-
-  return NULL;
-}
-
 static void
 m68k_linux_init_abi (struct gdbarch_info info, struct gdbarch *gdbarch)
 {
@@ -355,7 +346,7 @@ m68k_linux_init_abi (struct gdbarch_info
 
   set_gdbarch_decr_pc_after_break (gdbarch, 2);
 
-  frame_unwind_append_sniffer (gdbarch, m68k_linux_sigtramp_frame_sniffer);
+  frame_unwind_append_unwinder (gdbarch, &m68k_linux_sigtramp_frame_unwind);
 
   /* Shared library handling.  */
 

-- 
Andreas Schwab, SuSE Labs, schwab@suse.de
SuSE Linux Products GmbH, Maxfeldstraße 5, 90409 Nürnberg, Germany
PGP key fingerprint = 58CA 54C7 6D53 942B 1756  01D3 44D5 214B 8276 4ED5
"And now for something completely different."


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]